@@ -38,27 +38,25 @@
 #include "seahorse-gconf.h"
 #include "seahorse-gtkstock.h"
 #include "seahorse-context.h"
+#include "seahorse-unix-signal.h"
 
-static gboolean g_daemonize = TRUE;
-static gboolean g_quit = FALSE;
+static gboolean daemon_no_daemonize = FALSE;
+static gboolean daemon_running = FALSE;
+static gboolean daemon_quit = FALSE;
 
 static const gchar *daemon_icons[] = {
     SEAHORSE_ICON_SHARING,
     NULL
 };
 
-
-static const struct poptOption options[] = {
-	{ "no-daemonize", 'd', POPT_ARG_NONE | POPT_ARG_VAL, &g_daemonize, FALSE,
-	  N_("Do not daemonize seahorse-daemon"), NULL },
-
-	POPT_AUTOHELP
-	
-	POPT_TABLEEND
+static const GOptionEntry options[] = {
+    { "no-daemonize", 'd', 0, G_OPTION_ARG_NONE, &daemon_no_daemonize, 
+        N_("Do not daemonize seahorse-daemon"), NULL },
+    { NULL }
 };
 
 static void
-daemonize (const gchar **exec)
+daemonize ()
 {
     /* 
      * We can't use the normal daemon call, because we have
@@ -68,7 +66,7 @@ daemonize (const gchar **exec)
     pid_t pid;
     int i;
 
-    if (g_daemonize) {
+    if (!daemon_no_daemonize) {
         switch ((pid = fork ())) {
         case -1:
             err (1, _("couldn't fork process"));
@@ -99,41 +97,16 @@ daemonize (const gchar **exec)
     }
 
     /* The parent process or not daemonizing ... */
-
-    if (g_daemonize) {
-
-        /* If we were asked to exec another program, do that here */
-        if (!exec || !exec[0])
-            exit (0);
-
-        execvp (exec[0], (char**)exec);
-	    g_critical ("couldn't exec %s: %s\n", exec[0], strerror (errno));
-	    exit (1);
-
-    } else {
-
-        /* We can't overlay our process with the exec one if not daemonizing */
-        if (exec && exec[0])
-            g_warning ("cannot execute process when not daemonizing: %s", exec[0]);    
-
-    }
+    if (!daemon_no_daemonize)
+        exit (0);
 }
 
 static void
-on_quit (int signal)
+unix_signal (int signal)
 {
-    g_quit = 1;
-}
-
-static gboolean
-check_quit (gpointer data)
-{
-    if (g_quit) {
+    daemon_quit = TRUE;
+    if (daemon_running)
         gtk_main_quit ();
-        return FALSE;
-    }
-
-    return TRUE;
 }
 
 static void
@@ -202,9 +175,8 @@ client_die ()
 int main(int argc, char* argv[])
 {
     SeahorseOperation *op;
-    GnomeProgram *program = NULL;
     GnomeClient *client = NULL;
-    const char **args = NULL;
+    GOptionContext *octx = NULL;
 
     seahorse_secure_memory_init (65536);
     
@@ -222,8 +194,11 @@ int main(int argc, char* argv[])
 #endif
         err (1, _("couldn't drop privileges properly"));
     
-    program = gnome_program_init("seahorse-daemon", VERSION, LIBGNOMEUI_MODULE, argc, argv,
-                    GNOME_PARAM_POPT_TABLE, options,
+    octx = g_option_context_new ("");
+    g_option_context_add_main_entries (octx, options, GETTEXT_PACKAGE);
+
+    gnome_program_init ("seahorse-daemon", VERSION, LIBGNOMEUI_MODULE, argc, argv,
+                    GNOME_PARAM_GOPTION_CONTEXT, octx,
                     GNOME_PARAM_HUMAN_READABLE_NAME, _("Encryption Daemon (Seahorse)"),
                     GNOME_PARAM_APP_DATADIR, DATA_DIR, NULL);
 
@@ -231,14 +206,14 @@ int main(int argc, char* argv[])
      * All functions after this point have to print messages 
      * nicely and not just called exit() 
      */
-    daemonize (args);
+    daemonize ();
 
     /* Handle some signals */
-    signal (SIGINT, on_quit);
-    signal (SIGTERM, on_quit);
+    seahorse_unix_signal_register (SIGINT, unix_signal);
+    seahorse_unix_signal_register (SIGTERM, unix_signal);
 
     /* Force gconf to reconnect after daemonizing */
-    if (g_daemonize)
+    if (!daemon_no_daemonize)
         seahorse_gconf_disconnect ();    
         
     client = gnome_master_client();
@@ -264,8 +239,8 @@ int main(int argc, char* argv[])
 #endif
 
     /* Sometimes we've already gotten a quit signal */
-    if(!g_quit) {
-        g_timeout_add (100, check_quit, NULL);
+    if(!daemon_quit) {
+        daemon_running = TRUE;
         gtk_main ();
         g_message ("left gtk_main\n");
     }
